package main

import (
	"context"
	"log/slog"

	"koti.casa/numenor-labs/dsfx/cmd/dsfxnode/node"
	"koti.casa/numenor-labs/dsfx/pkg/disk"
	"koti.casa/numenor-labs/dsfx/pkg/storage/scoped"
	"koti.casa/numenor-labs/dsfx/pkg/system"
)

func main() {
	ctx := context.Background()

	sys := system.Default()

	configDir := sys.GetEnv("DSFXNODE_CONFIG_DIR")
	if configDir == "" {
		configDir = "/etc/dsfxnode/config"
	}
	configScope := scoped.New(disk.Default(), configDir)

	err := node.New(configScope, sys).Run(ctx)
	if err != nil {
		// Log the error and exit with a non-zero status code.
		slog.Error("Error running dsfxnode", slog.Any("error", err))
		sys.Exit(1)
	}
}
